@@ -287,6 +287,8 @@ def get_outbound_entity(entity: BaseEntity, private_key: RsaKey):
         # in all situations but is apparently being removed.
         # TODO: remove this once Diaspora removes the extra signature
         outbound.parent_signature = outbound.signature
+    if hasattr(outbound, "pre_send"):
+        outbound.pre_send()
     # Validate the entity
     outbound.validate(direction="outbound")
     return outbound
