@@ -1,11 +1,11 @@
 import pytest
 # noinspection PyPackageRequirements
 from freezegun import freeze_time
+from unittest.mock import patch
 
-from federation.entities.activitypub.entities import (
-    ActivitypubPost, ActivitypubAccept, ActivitypubFollow, ActivitypubProfile, ActivitypubComment,
-    ActivitypubRetraction, ActivitypubShare, ActivitypubImage)
-from federation.entities.base import Profile, Post
+from federation.entities.activitypub.mappers import get_outbound_entity
+import federation.entities.activitypub.models as models
+from federation.entities.base import Profile, Post, Comment, Retraction
 from federation.entities.diaspora.entities import (
     DiasporaPost, DiasporaComment, DiasporaLike, DiasporaProfile, DiasporaRetraction,
     DiasporaContact, DiasporaReshare,
@@ -18,8 +18,8 @@ from federation.tests.fixtures.payloads import DIASPORA_PUBLIC_PAYLOAD
 @pytest.fixture
 def activitypubannounce():
     with freeze_time("2019-08-05"):
-        return ActivitypubShare(
-            activity_id="http://127.0.0.1:8000/post/123456/#create",
+        return models.Announce(
+            id="http://127.0.0.1:8000/post/123456/#create",
             actor_id="http://127.0.0.1:8000/profile/123456/",
             target_id="http://127.0.0.1:8000/post/012345/",
         )
@@ -28,7 +28,7 @@ def activitypubannounce():
 @pytest.fixture
 def activitypubcomment():
     with freeze_time("2019-04-27"):
-        return ActivitypubComment(
+        obj = models.Comment(
             raw_content="raw_content",
             public=True,
             provider_display_name="Socialhome",
@@ -37,11 +37,13 @@ def activitypubcomment():
             actor_id=f"http://127.0.0.1:8000/profile/123456/",
             target_id="http://127.0.0.1:8000/post/012345/",
         )
+        obj.times={'edited':False, 'created':obj.created_at}
+        return obj
 
 
 @pytest.fixture
 def activitypubfollow():
-    return ActivitypubFollow(
+    return models.Follow(
         activity_id="https://localhost/follow",
         actor_id="https://localhost/profile",
         target_id="https://example.com/profile",
@@ -50,18 +52,18 @@ def activitypubfollow():
 
 @pytest.fixture
 def activitypubaccept(activitypubfollow):
-    return ActivitypubAccept(
+    return models.Accept(
         activity_id="https://localhost/accept",
         actor_id="https://localhost/profile",
         target_id="https://example.com/follow/1234",
-        object=activitypubfollow.to_as2(),
+        object_=activitypubfollow,
     )
 
 
 @pytest.fixture
 def activitypubpost():
     with freeze_time("2019-04-27"):
-        return ActivitypubPost(
+        obj = models.Post(
             raw_content="# raw_content",
             public=True,
             provider_display_name="Socialhome",
@@ -69,13 +71,17 @@ def activitypubpost():
             activity_id=f"http://127.0.0.1:8000/post/123456/#create",
             actor_id=f"http://127.0.0.1:8000/profile/123456/",
             _media_type="text/markdown",
+            to=["https://www.w3.org/ns/activitystreams#Public"],
+            cc=["https://http://127.0.0.1:8000/profile/123456/followers/"]
         )
+        obj.times={'edited':False, 'created':obj.created_at}
+        return obj
 
 
 @pytest.fixture
 def activitypubpost_diaspora_guid():
     with freeze_time("2019-04-27"):
-        return ActivitypubPost(
+        obj = models.Post(
             raw_content="raw_content",
             public=True,
             provider_display_name="Socialhome",
@@ -84,12 +90,14 @@ def activitypubpost_diaspora_guid():
             actor_id=f"http://127.0.0.1:8000/profile/123456/",
             guid="totallyrandomguid",
         )
+        obj.times={'edited':False, 'created':obj.created_at}
+        return obj
 
 
 @pytest.fixture
 def activitypubpost_images():
     with freeze_time("2019-04-27"):
-        return ActivitypubPost(
+        obj = models.Post(
             raw_content="raw_content",
             public=True,
             provider_display_name="Socialhome",
@@ -97,34 +105,38 @@ def activitypubpost_images():
             activity_id=f"http://127.0.0.1:8000/post/123456/#create",
             actor_id=f"http://127.0.0.1:8000/profile/123456/",
             _children=[
-                ActivitypubImage(url="foobar", media_type="image/jpeg"),
-                ActivitypubImage(url="barfoo", name="spam and eggs", media_type="image/jpeg"),
+                models.Image(url="foobar", media_type="image/jpeg"),
+                models.Image(url="barfoo", name="spam and eggs", media_type="image/jpeg"),
             ],
         )
+        obj.times={'edited':False, 'created':obj.created_at}
+        return obj
 
 
 @pytest.fixture
 def activitypubpost_mentions():
     with freeze_time("2019-04-27"):
-        return ActivitypubPost(
-            raw_content="""# raw_content\n\n@{someone@localhost.local} @{http://localhost.local/someone}""",
+        obj = models.Post(
+            raw_content="""# raw_content\n\n@someone@localhost.local @jaywink@localhost.local""",
             public=True,
             provider_display_name="Socialhome",
             id=f"http://127.0.0.1:8000/post/123456/",
             activity_id=f"http://127.0.0.1:8000/post/123456/#create",
             actor_id=f"http://127.0.0.1:8000/profile/123456/",
-            _mentions={
-                "http://127.0.0.1:8000/profile/999999",
-                "jaywink@localhost.local",
-                "http://localhost.local/someone",
-            }
+#            _mentions={
+#                "http://127.0.0.1:8000/profile/999999",
+#                "jaywink@localhost.local",
+#                "http://localhost.local/someone",
+#            }
         )
+        obj.times={'edited':False, 'created':obj.created_at}
+        return obj
 
 
 @pytest.fixture
 def activitypubpost_tags():
     with freeze_time("2019-04-27"):
-        return ActivitypubPost(
+        obj = models.Post(
             raw_content="# raw_content\n#foobar\n#barfoo",
             public=True,
             provider_display_name="Socialhome",
@@ -132,12 +144,14 @@ def activitypubpost_tags():
             activity_id=f"http://127.0.0.1:8000/post/123456/#create",
             actor_id=f"http://127.0.0.1:8000/profile/123456/",
         )
+        obj.times={'edited':False, 'created':obj.created_at}
+        return obj
 
 
 @pytest.fixture
 def activitypubpost_embedded_images():
     with freeze_time("2019-04-27"):
-        return ActivitypubPost(
+        obj = models.Post(
             raw_content="""
 #Cycling #lauttasaari #sea #sun
 
@@ -158,60 +172,69 @@ https://jasonrobinson.me/media/uploads/2019/07/16/daa24d89-cedf-4fc7-bad8-74a902
             activity_id=f"http://127.0.0.1:8000/post/123456/#create",
             actor_id=f"https://jasonrobinson.me/u/jaywink/",
         )
+        obj.times={'edited':False, 'created':obj.created_at}
+        return obj
 
 
 @pytest.fixture
-def activitypubprofile():
-    return ActivitypubProfile(
-        id="https://example.com/bob", raw_content="foobar", name="Bob Bobertson", public=True,
-        tag_list=["socialfederation", "federation"], image_urls={
-            "large": "urllarge", "medium": "urlmedium", "small": "urlsmall"
-        }, inboxes={
-            "private": "https://example.com/bob/private",
-            "public": "https://example.com/public",
-        }, public_key=PUBKEY, url="https://example.com/bob-bobertson"
-    )
+@patch.object(models.base.Image, 'get_media_type', return_value="image/jpeg")
+def activitypubprofile(mock_fetch):
+    with freeze_time("2022-09-06"):
+        return models.Person(
+            id="https://example.com/bob", raw_content="foobar", name="Bob Bobertson", public=True,
+            tag_list=["socialfederation", "federation"], image_urls={
+                "large": "urllarge", "medium": "urlmedium", "small": "urlsmall"
+            }, inboxes={
+                "private": "https://example.com/bob/private",
+                "public": "https://example.com/public",
+                }, public_key=PUBKEY, url="https://example.com/bob-bobertson"
+        )
 
 
 @pytest.fixture
-def activitypubprofile_diaspora_guid():
-    return ActivitypubProfile(
-        id="https://example.com/bob", raw_content="foobar", name="Bob Bobertson", public=True,
-        tag_list=["socialfederation", "federation"], image_urls={
-            "large": "urllarge", "medium": "urlmedium", "small": "urlsmall"
-        }, inboxes={
-            "private": "https://example.com/bob/private",
-            "public": "https://example.com/public",
-        }, public_key=PUBKEY, url="https://example.com/bob-bobertson",
-        guid="totallyrandomguid", handle="bob@example.com",
-    )
+@patch.object(models.base.Image, 'get_media_type', return_value="image/jpeg")
+def activitypubprofile_diaspora_guid(mock_fetch):
+    with freeze_time("2022-09-06"):
+        return models.Person(
+            id="https://example.com/bob", raw_content="foobar", name="Bob Bobertson", public=True,
+            tag_list=["socialfederation", "federation"], image_urls={
+                "large": "urllarge", "medium": "urlmedium", "small": "urlsmall"
+            }, inboxes={
+                "private": "https://example.com/bob/private",
+                "public": "https://example.com/public",
+            }, public_key=PUBKEY, url="https://example.com/bob-bobertson",
+            guid="totallyrandomguid", handle="bob@example.com",
+        )
 
 
 @pytest.fixture
 def activitypubretraction():
     with freeze_time("2019-04-27"):
-        return ActivitypubRetraction(
+        obj = Retraction(
             target_id="http://127.0.0.1:8000/post/123456/",
             activity_id="http://127.0.0.1:8000/post/123456/#delete",
             actor_id="http://127.0.0.1:8000/profile/123456/",
             entity_type="Post",
         )
+        return get_outbound_entity(obj, None)
 
 
 @pytest.fixture
 def activitypubretraction_announce():
     with freeze_time("2019-04-27"):
-        return ActivitypubRetraction(
-            target_id="http://127.0.0.1:8000/post/123456/activity",
+        obj = Retraction(
+            id="http://127.0.0.1:8000/post/123456/activity",
+            target_id="http://127.0.0.1:8000/post/123456",
             activity_id="http://127.0.0.1:8000/post/123456/#delete",
             actor_id="http://127.0.0.1:8000/profile/123456/",
             entity_type="Share",
         )
+        return get_outbound_entity(obj, None)
 
 
 @pytest.fixture
 def activitypubundofollow():
-    return ActivitypubFollow(
+    return models.Follow(
         activity_id="https://localhost/undo",
         actor_id="https://localhost/profile",
         target_id="https://example.com/profile",
@@ -232,7 +255,7 @@ def profile():
         inboxes={
             "private": "https://example.com/bob/private",
             "public": "https://example.com/public",
-        }, public_key=PUBKEY,
+        }, public_key=PUBKEY, to=["https://www.w3.org/ns/activitystreams#Public"]
     )
 
 
@@ -380,7 +403,7 @@ def diasporaretraction():
 
 @pytest.fixture
 def post():
-    return Post(
+    return models.Post(
         raw_content="""One more test before sleep 😅 This time with an image.
 
 ![](https://jasonrobinson.me/media/uploads/2020/12/27/1b2326c6-554c-4448-9da3-bdacddf2bb77.jpeg)""",
